    ;-----------------------------------;
    ; Encryptor/Decryptor, Version 2.0  ;
    ; Paul Brennick                     ;
    ; pebrennick@juno.com               ;
    ;-----------------------------------;
    .386                                ; Minimum requirement
    .model  flat, stdcall               ; 32 bit memory model
    option  casemap :none               ; Case sensitive

    include \GeneSys\include\windows.inc
    include \GeneSys\include\user32.inc
    include \GeneSys\include\kernel32.inc
    include \GeneSys\include\comctl32.inc
    include \GeneSys\include\comdlg32.inc

    includelib  \GeneSys\lib\user32.lib
    includelib  \GeneSys\lib\kernel32.lib
    includelib  \GeneSys\lib\comctl32.lib
    includelib  \GeneSys\lib\comdlg32.lib

    EncoderProc PROTO :DWORD, :DWORD, :DWORD, :DWORD
    Processing  PROTO

    .const
    ;-----------------------------------
    CHUNK_SIZE          equ 1024
    IDM_FILE_EXIT       equ 100
    IDM_HELP_ABOUT      equ 900
    IDC_INPUT_EDIT      equ 1000
    IDC_OUTPUT_EDIT     equ 1001
    IDC_KEY_EDIT        equ 1002
    IDC_ENC_COMBO       equ 1003
    IDC_BROWSE1_BUTTON  equ 1005
    IDC_BROWSE2_BUTTON  equ 1006
    IDC_GO_BUTTON       equ 1007
    IDC_EXIT_BUTTON     equ 1008
    IDD_ENCRYPT         equ 1009

    inv                 equ invoke

    .data
    ;-----------------------------------
    hInst           dd  0
    DlgMain         db  'Special', 0
    name_buffer     db  256 dup (0)
    curPosition     dd  0
    total_size      dd  0
    read_count      dd  0
    enc_dec         db  0
    key_check_sum   db  0
    key_check_sum2  db  0
    key_length      dd  0
    key             db  256 dup (0)
    chunk_location  dd  0
    chunk_buffer    db  CHUNK_SIZE dup (0)
    input_handle    dd  0
    output_handle   dd  0
    error_string    db  'Error...', 0
    key_error       db  'Error, no key has been provided!', 0
    input_error     db  'Error opening input file!', 0
    output_error    db  'Error creating output file!', 0
    about_title     db  'About...', 0
    about_string    db  '2048 Bit Key File Encryptor', 13, 10
                    db  '(c) 2006 by Paul Brennick', 13, 10
                    db  'This is a Freeware Product', 0
    done_title      db  'The Job is ...', 0
    done_string     db  'Finished!', 0
    num_enc_types   dd  2
    enc_type1       db  'Encrypt-Default Method', 0
    enc_type2       db  'Decrypt-Default Method', 0
    enc_table       dd  offset enc_type2
                    dd  offset enc_type1

.code
;---------------------------------------;
start:                                  ;
    inv  GetModuleHandleA, NULL         ; Get our app Handle from Windows
    mov  hInst, eax                     ; Store it for later use
    inv  InitCommonControls             ; Initialize the commonly used dialog boxes
;   ------------------------------------;
;   Call the resource dialog box        ;
;   ------------------------------------;
    inv  DialogBoxParam, hInst, ADDR DlgMain, 0, ADDR EncoderProc, 0
    inv  ExitProcess, eax               ; The application will exit here

EncoderProc proc    hWin:dword, uMsg:dword, wParam:dword, lParam:dword
;---------------------------------------;
    LOCAL ofn:OPENFILENAME              ; Structure used to open a file
                                        ;
    pushad                              ; Push all general-purpose registers
    mov  eax, [uMsg]                    ; Look for a message
    cmp  eax, WM_INITDIALOG             ; Is this the first pass?
    je   Initialize                     ; Jump if yes
    cmp  eax, WM_CLOSE                  ; Was the white X in the red box clicked?
    je   Exit_Button                    ; Jump if yes
    cmp  eax, WM_COMMAND                ; Was a button clicked or a menu item selected?
    je   Handle_Command                 ; Jump if yes
    jmp  Done_Not_Handled               ; Go look for more messages
Handle_Command:
    mov  eax, wParam                    ; Get possible message
    mov  bx, ax                         ; Save possible message
    shr  eax, 16                        ; Notifier is in the high word
    cmp  ax, 0                          ; Menu or Control=0, Accelerator=1
    jne  Done_Not_Handled               ; Jump if not a menu command
    cmp  bx, IDM_FILE_EXIT              ; 'Exit' selected from the Menu?
    je   Exit_Button                    ; Process the command, if yes
    cmp  bx, IDC_EXIT_BUTTON            ; 'Exit' Button pressed?
    je   Exit_Button                    ; Process the command, if yes
    cmp  bx, IDM_HELP_ABOUT             ; 'Help' selected from the Menu?
    je   about_enc                      ; Process the command, if yes
    cmp  bx, IDC_BROWSE1_BUTTON         ; Browse for Input File?
    je   Browse1_Button                 ; Process the command, if yes
    cmp  bx, IDC_BROWSE2_BUTTON         ; Browse for Output File?
    je   Browse2_Button                 ; Process the command, if yes
    cmp  bx, IDC_GO_BUTTON              ; Encrypt or Decrypt
    je   go_button                      ; Process the command, if yes
    jmp  Done_Not_Handled               ; Ignore all else
about_enc:
    inv  MessageBoxA, hWin, offset about_string, offset about_title, MB_OK
    xor  eax, eax
    ret
go_button:
    inv  GetDlgItem, hWin, IDC_KEY_EDIT ; Get handle to the edit window
    inv  GetWindowTextA, eax, offset key, 256   ; Get the Encrypt/Decrypt Key
    cmp  key[0], 0                      ; Empty?
    jne  proceed                        ; Jump if there is a key
    inv  MessageBoxA, hWin, offset key_error, offset error_string, MB_OK    ; Show the error
    jmp  Done_Not_Handled               ; Abort the task, return to the loop
proceed:
    inv  GetDlgItem, hWin, IDC_INPUT_EDIT   ; Get handle to the edit window
    inv  GetWindowTextA, eax, offset name_buffer, 256   ; Get the File to Encrypt or Decrypt
;   Open the existing file for reading and map it to a buffer
    inv  CreateFileA, offset name_buffer, GENERIC_READ, 0, 0, OPEN_EXISTING, 0, 0
    cmp  eax, -1                        ; Error opening?
    jne  good_open_input                ; Jump if file opened with no problems
    inv  MessageBoxA, hWin, offset input_error, offset error_string, MB_OK  ; Show error
    jmp  Done_Handled                   ; Abort the task, return to the loop
good_open_input:
    mov  [input_handle], eax            ; Save the handle
    inv  GetDlgItem, hWin, IDC_OUTPUT_EDIT  ; Get handle to the edit window
    inv  GetWindowTextA, eax, offset name_buffer, 256   ; Get the File that will be written to
;   Open the existing file for Writing and map it to a buffer
    inv  CreateFileA, offset name_buffer, GENERIC_WRITE, 0, 0, CREATE_ALWAYS, FILE_ATTRIBUTE_NORMAL, 0
    cmp  eax, -1                        ; Error opening?
    jne  good_open_output               ; Jump if file opened with no problems
    inv  CloseHandle, [input_handle]    ; Close the file already opened
    inv  MessageBoxA, hWin, offset output_error, offset error_string, MB_OK ; Show the error
    jmp  Done_Handled                   ; Abort the task, return to the loop
good_open_output:
    mov  [output_handle], eax           ; Save the handle
    mov  [curPosition], 0               ; Set the File Pointer to the beginning
    inv  GetFileSize, [input_handle], 0 ; Get the amount of bytes to encode
    mov  [total_size], eax              ; Save it
    inv  SendDlgItemMessageA, hWin, IDC_ENC_COMBO, CB_GETCURSEL, 0, 0   ; Get index of item in window
    mov  [enc_dec], al                  ; Save it
    mov  edi, offset key                ; Get pointer to the Key buffer
    mov  ecx, 64                        ; Get amount of iterations
    xor  eax, eax                       ; clear the register
    cld                                 ; Set to move forward
    rep  stosd                          ; Clear 256 bytes
    inv  GetDlgItem, hWin, IDC_KEY_EDIT ; Get handle to the edit window
    inv  GetWindowTextA, eax, offset key, 256   ; Get the Key into the Key buffer
    mov  byte ptr [key_check_sum2], 0   ; Start with CheckSum=0
    mov  esi, offset key                ; Get a pointer to the Key buffer
    xor  ebx, ebx                       ; Clear the register
    mov  edx, 256                       ; Get the size of the Key buffer
make_check_sum:
    lodsb                               ; Get a byte
    add  [key_check_sum2], al           ; Accumulate it into the CheckSum
    mov  cl, al
    ror  bl, cl
    ror  byte ptr [key_check_sum2], cl
    xor  bl, al
    dec  edx                            ; Count the pass
    jnz  make_check_sum                 ; Do it 256 times
    mov  [key_check_sum], bl            ; Save the overflow
    mov  edi, offset key                ; Get a Pointer to the Key buffer
    xor  al, al                         ; Zero is the comparison value
    mov  cx, 256                        ; Amount of iterations
    cld                                 ; Move forward through the buffer
    repne scasb                         ; Keep searching for the zero
    dec  edi                            ; Minus 1
    sub  edi, offset key                ; Calculate the length (EDI contains amt. of zeroes)
    mov  [key_length], edi              ; Save the length of the Key
    inv  Processing                     ; Go Encrypt or Decrypt
;   Report that the task has been completed (one way or the other)
    inv  MessageBoxA, 0, offset done_string, offset done_title, MB_OK
    jmp  Done_Handled                   ; The task is completed, return to the loop
Browse1_Button:
    mov  [ofn.lStructSize], 76          ; Size of structure in bytes
    mov  eax, hWin                      ; Get dialog box handle
    mov  [ofn.hwndOwner], eax
    mov  [ofn.hInstance], 0
    mov  [ofn.lpstrFilter], 0
    mov  [ofn.lpstrCustomFilter], 0
    mov  [ofn.nFilterIndex], 0
    mov  eax, offset name_buffer        ; Filename goes into this buffer
    mov  byte ptr [eax], 0              ; Zero the buffer
    mov  [ofn.lpstrFile], eax           ; Save the buffer pointer
    mov  [ofn.nMaxFile], 256            ; Set max. characters to get from edit box
    mov  [ofn.lpstrFileTitle], 0
    mov  [ofn.lpstrInitialDir], 0
    mov  [ofn.lpstrTitle], 0
    mov  [ofn.Flags], OFN_EXPLORER or OFN_FILEMUSTEXIST or OFN_NOCHANGEDIR or OFN_PATHMUSTEXIST
    mov  [ofn.lpstrDefExt], 0
    inv  GetOpenFileNameA, addr ofn     ; Open File Dialog to get a filename
    or   eax, eax                       ; Anything selected?
    jz   Done_Handled                   ; Jump if no
    inv  GetDlgItem, hWin, IDC_INPUT_EDIT           ; Get Handle to the edit box
    inv  SetWindowTextA, eax, offset name_buffer    ; Store the filename in it
    jmp  Done_Handled                   ; Back to the message loop
Browse2_Button:
    mov  [ofn.lStructSize], 76          ; Size of structure in bytes
    mov  eax, hWin                      ; Get dialog box handle
    mov  [ofn.hwndOwner], eax
    mov  [ofn.hInstance], 0
    mov  [ofn.lpstrFilter], 0
    mov  [ofn.lpstrCustomFilter], 0
    mov  [ofn.nFilterIndex], 0
    mov  eax, offset name_buffer        ; Filename goes into this buffer
    mov  byte ptr [eax], 0              ; Zero the buffer
    mov  [ofn.lpstrFile], eax           ; Save the buffer pointer
    mov  [ofn.nMaxFile], 256            ; Set max. characters to get from edit box
    mov  [ofn.lpstrFileTitle], 0
    mov  [ofn.lpstrInitialDir], 0
    mov  [ofn.lpstrTitle], 0
    mov  [ofn.Flags], OFN_EXPLORER or OFN_NOCHANGEDIR
    mov  [ofn.lpstrDefExt], 0
    inv  GetOpenFileNameA, addr ofn     ; Open File Dialog to get a filename
    or   eax, eax                       ; Anything selected?
    jz   Done_Handled                   ; Jump if no
    inv  GetDlgItem, hWin, IDC_OUTPUT_EDIT          ; Get Handle to the edit box
    inv  SetWindowTextA, eax, offset name_buffer    ; Store the filename in it
    jmp  Done_Handled                   ; Back to the message loop
Exit_Button:
    inv  PostQuitMessage, 0             ; Exit the Application via the message loop
    jmp  Done_Handled                   ; Back to the message loop
Initialize:
    mov  ecx, [num_enc_types]           ; Get the amount of entries 
add_string_loop:
    push ecx                            ; Save it
    lea  edx, [ecx-1]                   ; Make it zero based
    mov  edx, [edx*4+offset enc_table]  ; Point to a table entry
    inv  SendDlgItemMessageA, hWin, IDC_ENC_COMBO, CB_ADDSTRING, 0, edx ; Store it in the list
    pop  ecx                            ; Restore the count
    loop add_string_loop                ; Get all the entries
    inv  SendDlgItemMessageA, hWin, IDC_ENC_COMBO, CB_SETCURSEL, 0, 0   ; set default
    jmp  Done_Handled                   ; Back to the message loop
Done_Handled:
    popad                               ; Retore general registers
    xor  eax, eax
    inc  eax        
    ret                                 ; Return to the message loop
Done_Not_Handled:
    popad                               ; Retore general registers
    xor  eax, eax
    ret                                 ; Return to the message loop
;---------------------------------------
EncoderProc endp

Processing  proc
;---------------------------------------
inProcess:
    inv  ReadFile, [input_handle], offset chunk_buffer, CHUNK_SIZE, offset read_count, 0
    mov  [chunk_location], 0            ; Set the location pointer to the start of the buffer
    mov  ecx, [read_count]              ; The amount of characters to process
    or   ecx, ecx                       ; Is the task completed?
    jz   done_file                      ; jump if done
chunk_loop:
    push ecx                            ; Preserve the character counter
    xor  edx, edx                       ; Prepare for next pass
    mov  eax, [curPosition]             ; Offset from the beginning of the file
    mov  ebx, [key_length]              ; Length of your private key
    div  ebx                            ; Calculate offset into the mask
    mov  al, [key+edx]                  ; Get mask character
    push eax                            ; Save it
    cmp  [enc_dec], 0                   ; 0 [or default] = Encrypt]
    je   encrypt
;---------------------------------------;
;   The DeCrypt Algorythm starts here   ;
;---------------------------------------;
    mov  ax, word ptr [curPosition]     ; Get offset within the file
    mov  cx, ax
    mov  bx, ax
    add  cl, ch
    mov  ebp, [chunk_location]          ; Pointer within the 1024 byte chunk buffer
    add  ebp, offset chunk_buffer       ; Calculate the position from the start of the buffer
    mov  dl, [ebp]                      ; Get character to be processed
    xor  dl, [key_check_sum]
    xchg [key_check_sum2], cl
    ror  dl, cl
    xchg [key_check_sum2], cl
    rol  dl, cl
    add  dl, cl
    ror  bl, 3
    xor  dl, bl
    and  al, ah
    xor  dl, al
    neg  dl
    pop  eax                            ; Restore mask character
    xor  dl, al                         ; Mask the character we are processing
    mov  [ebp], dl                      ; Put the processed character back into the buffer
    jmp  next_byte                      ; Go get another character
;---------------------------------------;
;   The EnCrypt Algorythm starts here   ;
;---------------------------------------;
encrypt:
    pop  eax                            ; Restore mask character
    mov  ebp, [chunk_location]          ; Pointer within the 1024 byte chunk buffer
    add  ebp, offset chunk_buffer       ; Calculate the position from the start of the buffer
    mov  dl, [ebp]                      ; Get character to be processed
    xor  dl, al                         ; Unmask the character we are processing
    neg  dl
    mov  ax, word ptr [curPosition]     ; Get offset within the file
    mov  cx, ax
    mov  bx, ax
    and  al, ah
    xor  dl, al
    ror  bl, 3
    xor  dl, bl
    add  cl, ch
    sub  dl, cl
    ror  dl, cl
    xchg [key_check_sum2], cl
    rol  dl, cl
    xchg [key_check_sum2], cl
    xor  dl, [key_check_sum]
    mov  [ebp], dl                      ; Put the processed character back into the buffer
;---------------------------------------;
;   Both algos pass through this area   ;
;---------------------------------------;
next_byte:
    inc  [curPosition]                  ; Point to the next chunk within the file
    inc  [chunk_location]               ; Number of bytes to write
    pop  ecx                            ; Restore the character counter
    dec  ecx                            ; One less character to process
    jnz  chunk_loop                     ; Continue processing until done
    inv  WriteFile, [output_handle], offset chunk_buffer, [chunk_location], offset read_count, 0
    jmp  inProcess                      ; Go process another character
done_file:
    inv  CloseHandle, [input_handle]    ; Close the input file handle
    inv  CloseHandle, [output_handle]   ; Close the output file handle
    mov  dword ptr [input_handle], 0    ; Clear the handle
    mov  dword ptr [output_handle], 0   ; Clear handle
    ret
;---------------------------------------
Processing  endp

    end  start

